use std::{fs::File, sync::Mutex};

use rss::{ChannelBuilder, ItemBuilder};
use time::OffsetDateTime;

use crate::remaining_place::RemainingPlace;

pub fn update_rss_file(places: Vec<RemainingPlace>, url: &str, rss_file: &str) {
    let mut channel = ChannelBuilder::default()
        .title(String::from("LFS Restplatzbörse"))
        .link(url.to_string())
        .description(String::from(
            "Ein RSS Feed der Restplatzbörse der Landesfeuerwehrschule Sachsen. Nicht offiziell.",
        ))
        .language(Some("de-DE".to_string()))
        .build();

    let title = format!(
        "Restplatzbörse Update - {}",
        OffsetDateTime::now_local()
            .unwrap_or(OffsetDateTime::now_utc())
            .format(time::macros::format_description!(
                "[year]-[month]-[day] [hour]:[minute]"
            ))
            .unwrap()
    );

    let content = places
        .iter()
        .map(|place| {
            format!(
                "{} - {} - {} - {} Plätze</br>",
                place.id, place.description, place.date, place.free
            )
        })
        .collect::<Vec<String>>()
        .join("\n");

    let item = ItemBuilder::default()
        .title(Some(title))
        .content(Some(content))
        .build();

    channel.set_items(vec![item]);

    let output = File::create(rss_file).unwrap();
    channel.pretty_write_to(output, ' ' as u8, 2).unwrap();
}

pub fn should_feed_be_updated(
    new_places: &Vec<RemainingPlace>,
    last_places: &Mutex<Vec<RemainingPlace>>,
) -> bool {
    let mut last_places = last_places.lock().unwrap();

    let are_the_same_places = new_places.len() == last_places.len()
        && new_places
            .iter()
            .zip(last_places.iter())
            .all(|(one, two)| one == two);

    if !are_the_same_places {
        last_places.clear();
        last_places.append(&mut new_places.clone());
    }

    !are_the_same_places
}
